import re
from xml.etree.ElementTree import Element

from Dictionary.Word import Word
from MorphologicalAnalysis.MorphologicalParse import MorphologicalParse

from DependencyParser.Turkish.TurkishDependencyRelation import TurkishDependencyRelation


class TurkishDependencyTreeBankWord(Word):

    __parse: MorphologicalParse
    __originalParses: list
    __relation: TurkishDependencyRelation

    def __init__(self, wordNode: Element):
        """
        Given the parsed xml node which contains information about a word and related attributes including the
        dependencies, the method constructs a {@link TurkishDependencyTreeBankWord} from it.

        PARAMETERS
        -----------
        wordNode : Element
            Xml parsed node containing information about a word.
        """
        toWord = 0
        toIG = 0
        self.__originalParses = []
        self.name = wordNode.text
        IG = wordNode.attrib["IG"]
        IG = IG[:IG.index("+")] + "+" + IG[IG.index("+") + 1:].upper()
        self.__parse = MorphologicalParse(self.splitIntoInflectionalGroups(IG))
        self.__relation = None
        relationName = wordNode.attrib["REL"]
        if relationName != "[,( )]":
            relationParts = re.compile("[\\[()\\],]").split(relationName)
            index = 0
            for part in relationParts:
                if len(part) != 0:
                    index = index + 1
                    if index == 1:
                        toWord = int(part)
                    elif index == 2:
                        toIG = int(part)
                    elif index == 3:
                        self.__relation = TurkishDependencyRelation(toWord - 1, toIG - 1, part)
        for i in range(1, 10):
            if ("ORG_ID" + str(i)) in wordNode.attrib:
                IG = wordNode.attrib["ORG_ID" + str(i)]
                IG = IG[:IG.index("+")] + "+" + IG[IG.index("+") + 1:].upper()
                self.__originalParses.append(MorphologicalParse(self.splitIntoInflectionalGroups(IG)))

    def splitIntoInflectionalGroups(self, IG: str) -> list:
        """
        Given the morphological parse of a word, this method splits it into inflectional groups.

        PARAMETERS
        ----------
        IG : str
            Morphological parse of the word in string form.

        RETURNS
        -------
        list
            A list of inflectional groups stored as strings.
        """
        inflectionalGroups = []
        IG = IG.replace("(+Punc", "@").replace(")+Punc", "$")
        IGs = re.compile("[\\[()\\]]").split(IG)
        for part in IGs:
            part = part.replace("@", "(+Punc").replace("$", ")+Punc")
            if len(part) != 0:
                inflectionalGroups.append(part)
        return inflectionalGroups

    def getParse(self) -> MorphologicalParse:
        """
        Accessor for the parse attribute

        RETURNS
        -------
        MorphologicalPArse
            Parse attribute
        """
        return self.__parse

    def getOriginalParse(self, index: int) -> MorphologicalParse:
        """
        Accessor for a specific parse.

        PARAMETERS
        ----------
        index : int
            Index of the word.

        RETURNS
        -------
        MorphologicalParse
            Parse of the index'th word
        """
        if index < len(self.__originalParses):
            return self.__originalParses[index]
        else:
            return None

    def size(self) -> int:
        """
        Number of words in this item.

        RETURNS
        -------
        int
            Number of words in this item.
        """
        return len(self.__originalParses)

    def getRelation(self) -> TurkishDependencyRelation:
        """
        Accessor for the relation attribute.

        RETURNS
        -------
        TurkishDependencyRelation
            relation attribute.
        """
        return self.__relation



from DependencyParser.DependencyRelation import DependencyRelation
from DependencyParser.Stanford.StanfordDependencyType import StanfordDependencyType


class StanfordDependencyRelation(DependencyRelation):

    __stanfordDependencyType: StanfordDependencyType

    stanfordDependencyTypes = ["ACOMP", "ADVCL", "ADVMOD", "AGENT", "AMOD", "APPOS", "AUX", "AUXPASS", "CC", "CCOMP",
                               "CONJ", "COP", "CSUBJ", "CSUBJPASS", "DEP", "DET", "DISCOURSE", "DOBJ", "EXPL",
                               "GOESWITH", "IOBJ", "MARK", "MWE", "NEG", "NN", "NPADVMOD", "NSUBJ", "NSUBJPASS", "NUM",
                               "NUMBER", "PARATAXIS", "PCOMP", "POBJ", "PASS", "POSSESSIVE", "PRECONJ", "PREDET",
                               "PREP", "PREPC", "PRT", "PUNCT", "QUANTMOD", "RCMOD", "REF", "ROOT", "TMOD", "VMOD",
                               "XCOMP", "XSUBJ"]

    stanfordDependencyTags = [StanfordDependencyType.ACOMP, StanfordDependencyType.ADVCL,
                              StanfordDependencyType.ADVMOD, StanfordDependencyType.AGENT, StanfordDependencyType.AMOD,
                              StanfordDependencyType.APPOS, StanfordDependencyType.AUX,
                              StanfordDependencyType.AUXPASS, StanfordDependencyType.CC, StanfordDependencyType.CCOMP,
                              StanfordDependencyType.CONJ, StanfordDependencyType.COP,
                              StanfordDependencyType.CSUBJ, StanfordDependencyType.CSUBJPASS,
                              StanfordDependencyType.DEP, StanfordDependencyType.DET, StanfordDependencyType.DISCOURSE,
                              StanfordDependencyType.DOBJ, StanfordDependencyType.EXPL, StanfordDependencyType.GOESWITH,
                              StanfordDependencyType.IOBJ, StanfordDependencyType.MARK,
                              StanfordDependencyType.MWE, StanfordDependencyType.NEG, StanfordDependencyType.NN,
                              StanfordDependencyType.NPADVMOD, StanfordDependencyType.NSUBJ,
                              StanfordDependencyType.NSUBJPASS, StanfordDependencyType.NUM,
                              StanfordDependencyType.NUMBER, StanfordDependencyType.PARATAXIS,
                              StanfordDependencyType.PCOMP,
                              StanfordDependencyType.POBJ, StanfordDependencyType.POSS,
                              StanfordDependencyType.POSSESSIVE, StanfordDependencyType.PRECONJ,
                              StanfordDependencyType.PREDET,
                              StanfordDependencyType.PREP, StanfordDependencyType.PREPC, StanfordDependencyType.PRT,
                              StanfordDependencyType.PUNCT, StanfordDependencyType.QUANTMOD,
                              StanfordDependencyType.RCMOD, StanfordDependencyType.REF, StanfordDependencyType.ROOT,
                              StanfordDependencyType.TMOD, StanfordDependencyType.VMOD,
                              StanfordDependencyType.XCOMP, StanfordDependencyType.XSUBJ]

    @staticmethod
    def getDependencyTag(tag: str) -> StanfordDependencyType:
        """
        The getDependencyTag method takes an dependency tag as string and returns the {@link StanfordDependencyType}
        form of it.

        PARAMETERS
        ----------
        tag : str
            Type of the dependency tag in string form

        RETURNS
        -------
        StanfordDependencyType
            Type of the dependency in StanfordDependencyType form
        """
        for j in range(len(StanfordDependencyRelation.stanfordDependencyTags)):
            if tag.upper() == StanfordDependencyRelation.stanfordDependencyTypes[j]:
                return StanfordDependencyRelation.stanfordDependencyTags[j]
        return None

    def __init__(self, toWord: int, dependencyType: str = None):
        """
        Another constructor for StanfordDependencyRelation. Gets input toWord and dependencyType as arguments and
        calls the super class's constructor and sets the dependency type.

        PARAMETERS
        ----------
        toWord : int
            Index of the word in the sentence that dependency relation is related
        dependencyType : str
            Type of the dependency relation in string form
        """
        super().__init__(toWord)
        if dependencyType is not None:
            self.__stanfordDependencyType = StanfordDependencyRelation.getDependencyTag(dependencyType)

    def __str__(self) -> str:
        return self.__stanfordDependencyType.name



from __future__ import annotations

from Corpus.Corpus import Corpus
from DataStructure.CounterHashMap import CounterHashMap
from DependencyParser.ParserEvaluationScore import ParserEvaluationScore
from DependencyParser.Universal.UniversalDependencyTreeBankSentence import UniversalDependencyTreeBankSentence


class UniversalDependencyTreeBankCorpus(Corpus):

    def __init__(self, fileName: str):
        self.sentences = []
        self.paragraphs = []
        self.wordList = CounterHashMap()
        sentence = ""
        file = open(fileName, "r")
        lines = file.readlines()
        file.close()
        for line in lines:
            line = line.strip()
            if len(line) == 0:
                self.addSentence(UniversalDependencyTreeBankSentence(sentence))
                sentence = ""
            else:
                sentence = sentence + line + "\n"

    def compareParses(self, corpus: UniversalDependencyTreeBankCorpus) -> ParserEvaluationScore:
        score = ParserEvaluationScore()
        for i in range(len(self.sentences)):
            score.add(self.sentences[i].compareParses(corpus.getSentence(i)))
        return score



from Dictionary.Word import Word

from DependencyParser.Universal.UniversalDependencyPosType import UniversalDependencyPosType
from DependencyParser.Universal.UniversalDependencyRelation import UniversalDependencyRelation
from DependencyParser.Universal.UniversalDependencyTreeBankFeatures import UniversalDependencyTreeBankFeatures


class UniversalDependencyTreeBankWord(Word):

    id: int
    lemma: str
    upos: UniversalDependencyPosType
    xpos: str
    features: UniversalDependencyTreeBankFeatures
    relation: UniversalDependencyRelation
    deps: str
    misc: str

    def __init__(self, id: int, name: str, lemma: str, upos: UniversalDependencyPosType, xpos: str,
                 features: UniversalDependencyTreeBankFeatures, relation: UniversalDependencyRelation, deps: str,
                 misc: str):
        super().__init__(name)
        self.id = id
        self.lemma = lemma
        self.upos = upos
        self.xpos = xpos
        self.deps = deps
        self.features = features
        self.relation = relation
        self.misc = misc

    def getId(self) -> int:
        return self.id

    def getLemma(self) -> str:
        return self.lemma

    def getUpos(self) -> UniversalDependencyPosType:
        return self.upos

    def getXPos(self) -> str:
        return self.xpos

    def getFeatures(self) -> UniversalDependencyTreeBankFeatures:
        return self.features

    def getFeatureValue(self, featureName: str) -> str:
        return self.features.getFeatureValue(featureName)

    def featureExists(self, featureName: str) -> bool:
        return self.features.featureExists(featureName)

    def getRelation(self) -> UniversalDependencyRelation:
        return self.relation

    def getDeps(self) -> str:
        return self.deps

    def getMisc(self) -> str:
        return self.misc

    def __str__(self) -> str:
        return self.id.__str__() + "\t" + self.name + "\t" + self.lemma + "\t" + self.upos.__str__() + "\t" + \
               self.xpos + "\t" + self.features.__str__() + "\t" + self.relation.to().__str__() + "\t" + \
               self.relation.__str__().lower() + "\t" + self.deps + "\t" + self.misc



from __future__ import annotations

class ParserEvaluationScore:

    LAS: float
    UAS: float
    LS: float
    wordCount: int

    def __init__(self, LAS: float = 0.0, UAS: float = 0.0, LS : float = 0.0, wordCount : int = 0):
        self.LAS = LAS
        self.UAS = UAS
        self.LS = LS
        self.wordCount = wordCount

    def getLS(self) -> float:
        return self.LS

    def getLAS(self) -> float:
        return self.LAS

    def getUAS(self) -> float:
        return self.UAS

    def getWordCount(self) -> int:
        return self.wordCount

    def add(self, parserEvaluationScore: ParserEvaluationScore):
        self.LAS = (self.LAS * self.wordCount + parserEvaluationScore.LAS * parserEvaluationScore.wordCount) / \
                   (self.wordCount + parserEvaluationScore.wordCount)
        self.UAS = (self.UAS * self.wordCount + parserEvaluationScore.UAS * parserEvaluationScore.wordCount) / \
                   (self.wordCount + parserEvaluationScore.wordCount)
        self.LS = (self.LS * self.wordCount + parserEvaluationScore.LS * parserEvaluationScore.wordCount) / \
                  (self.wordCount + parserEvaluationScore.wordCount)
        self.wordCount += parserEvaluationScore.wordCount



class UniversalDependencyTreeBankFeatures:

    featureList: dict

    def __init__(self, features: str):
        self.featureList = {}
        if features != "_":
            _list = features.split("\\|")
            for feature in _list:
                if "=" in feature:
                    featureName = feature[0: feature.index("=") - 1].strip()
                    featureValue = feature[feature.index("=") + 1:].strip()
                    self.featureList[featureName] = featureValue
                else:
                    print("Feature does not contain = ->" + features)

    def getFeatureValue(self, feature: str) -> str:
        return self.featureList[feature]

    def featureExists(self, feature: str) -> bool:
        return feature in self.featureList

    def __str__(self) -> str:
        if len(self.featureList) == 0:
            return "_"
        result = ""
        for feature in self.featureList:
            if result == "":
                result = feature + "=" + self.getFeatureValue(feature)
            else:
                result += "|" + feature + "=" + self.getFeatureValue(feature)
        return result



from __future__ import annotations

from DependencyParser.DependencyRelation import DependencyRelation
from DependencyParser.ParserEvaluationScore import ParserEvaluationScore
from DependencyParser.Universal.UniversalDependencyPosType import UniversalDependencyPosType
from DependencyParser.Universal.UniversalDependencyType import UniversalDependencyType


class UniversalDependencyRelation(DependencyRelation):
    __universalDependencyType: UniversalDependencyType

    universalDependencyTypes = ["ACL", "ADVCL",
                                "ADVMOD", "AMOD",
                                "APPOS",
                                "AUX", "AUXPASS",
                                "CASE", "CC", "CCOMP",
                                "CLF", "COMPOUND",
                                "CONJ", "COP", "CSUBJ",
                                "DEP", "DET",
                                "DISCOURSE", "DISLOCATED",
                                "EXPL",
                                "FIXED", "FLAT",
                                "FOREIGN",
                                "GOESWITH",
                                "IOBJ",
                                "LIST",
                                "MARK",
                                "NMOD", "NSUBJ",
                                "NUMMOD",
                                "OBJ", "OBL", "ORPHAN",
                                "PARATAXIS", "PUNCT",
                                "REPARANDUM", "ROOT",
                                "VOCATIVE",
                                "XCOMP", "ACL:RELCL",
                                "AUX:PASS", "CC:PRECONJ",
                                "COMPOUND:PRT", "DET:PREDET",
                                "FLAT:FOREIGN", "NSUBJ:PASS",
                                "CSUBJ:PASS", "NMOD:NPMOD",
                                "NMOD:POSS", "NMOD:TMOD",
                                "ADVMOD:EMPH", "AUX:Q",
                                "COMPOUND:LVC", "COMPOUND:REDUP",
                                "CSUBJ:COP", "NMOD:COMP",
                                "NMOD:PART", "NSUBJ:COP",
                                "OBL:AGENT", "OBL:TMOD"]

    universalDependencyTags = [UniversalDependencyType.ACL, UniversalDependencyType.ADVCL,
                               UniversalDependencyType.ADVMOD, UniversalDependencyType.AMOD,
                               UniversalDependencyType.APPOS,
                               UniversalDependencyType.AUX, UniversalDependencyType.AUXPASS,
                               UniversalDependencyType.CASE, UniversalDependencyType.CC, UniversalDependencyType.CCOMP,
                               UniversalDependencyType.CLF, UniversalDependencyType.COMPOUND,
                               UniversalDependencyType.CONJ, UniversalDependencyType.COP, UniversalDependencyType.CSUBJ,
                               UniversalDependencyType.DEP, UniversalDependencyType.DET,
                               UniversalDependencyType.DISCOURSE, UniversalDependencyType.DISLOCATED,
                               UniversalDependencyType.EXPL,
                               UniversalDependencyType.FIXED, UniversalDependencyType.FLAT,
                               UniversalDependencyType.FOREIGN,
                               UniversalDependencyType.GOESWITH,
                               UniversalDependencyType.IOBJ,
                               UniversalDependencyType.LIST,
                               UniversalDependencyType.MARK,
                               UniversalDependencyType.NMOD, UniversalDependencyType.NSUBJ,
                               UniversalDependencyType.NUMMOD,
                               UniversalDependencyType.OBJ, UniversalDependencyType.OBL, UniversalDependencyType.ORPHAN,
                               UniversalDependencyType.PARATAXIS, UniversalDependencyType.PUNCT,
                               UniversalDependencyType.REPARANDUM, UniversalDependencyType.ROOT,
                               UniversalDependencyType.VOCATIVE,
                               UniversalDependencyType.XCOMP, UniversalDependencyType.ACL_RELCL,
                               UniversalDependencyType.AUX_PASS,
                               UniversalDependencyType.CC_PRECONJ, UniversalDependencyType.COMPOUND_PRT,
                               UniversalDependencyType.DET_PREDET, UniversalDependencyType.FLAT_FOREIGN,
                               UniversalDependencyType.NSUBJ_PASS, UniversalDependencyType.CSUBJ_PASS,
                               UniversalDependencyType.NMOD_NPMOD, UniversalDependencyType.NMOD_POSS,
                               UniversalDependencyType.NMOD_TMOD, UniversalDependencyType.ADVMOD_EMPH,
                               UniversalDependencyType.AUX_Q, UniversalDependencyType.COMPOUND_LVC,
                               UniversalDependencyType.COMPOUND_REDUP, UniversalDependencyType.CSUBJ_COP,
                               UniversalDependencyType.NMOD_COMP, UniversalDependencyType.NMOD_PART,
                               UniversalDependencyType.NSUBJ_COP, UniversalDependencyType.OBL_AGENT,
                               UniversalDependencyType.OBL_TMOD]

    universalDependencyPosTypes = ["ADJ", "ADV", "INTJ", "NOUN", "PROPN", "VERB", "ADP", "AUX", "CCONJ",
                                   "DET", "NUM", "PART", "PRON", "SCONJ", "PUNCT", "SYM", "X"]

    universalDependencyPosTags = [UniversalDependencyPosType.ADJ, UniversalDependencyPosType.ADV,
                                  UniversalDependencyPosType.INTJ, UniversalDependencyPosType.NOUN,
                                  UniversalDependencyPosType.PROPN,
                                  UniversalDependencyPosType.VERB, UniversalDependencyPosType.ADP,
                                  UniversalDependencyPosType.AUX, UniversalDependencyPosType.CCONJ,
                                  UniversalDependencyPosType.DET, UniversalDependencyPosType.NUM,
                                  UniversalDependencyPosType.PART,
                                  UniversalDependencyPosType.PRON, UniversalDependencyPosType.SCONJ,
                                  UniversalDependencyPosType.PUNCT, UniversalDependencyPosType.SYM,
                                  UniversalDependencyPosType.X]

    @staticmethod
    def getDependencyTag(tag: str) -> UniversalDependencyType:
        """
        The getDependencyTag method takes an dependency tag as string and returns the UniversalDependencyType
        form of it.

        PARAMETERS
        ----------
        tag : str
            Type of the dependency tag in string form

        RETURNS
        -------
        UniversalDependencyType
            Type of the dependency in UniversalDependencyType form
        """
        for j in range(len(UniversalDependencyRelation.universalDependencyTags)):
            if tag.upper() == UniversalDependencyRelation.universalDependencyTypes[j]:
                return UniversalDependencyRelation.universalDependencyTags[j]
        return None

    @staticmethod
    def getDependencyPosType(tag: str) -> UniversalDependencyPosType:
        for j in range(len(UniversalDependencyRelation.universalDependencyPosTypes)):
            if tag.upper() == UniversalDependencyRelation.universalDependencyPosTypes[j]:
                return UniversalDependencyRelation.universalDependencyPosTags[j]
        return None

    def __init__(self, toWord: int, dependencyType: str = None):
        """
        Overriden Universal Dependency Relation constructor. Gets toWord as input and calls it super class's constructor

        PARAMETERS
        ----------
        toWord : int
            Index of the word in the sentence that dependency relation is related
        """
        super().__init__(toWord)
        if dependencyType is not None:
            self.__universalDependencyType = UniversalDependencyRelation.getDependencyTag(dependencyType)

    def compareRelations(self, relation: UniversalDependencyRelation) -> ParserEvaluationScore:
        LS = 0.0
        LAS = 0.0
        UAS = 0.0
        if self.__str__() == relation.__str__():
            LS = 1.0
            if self.toWord == relation.to():
                LAS = 1.0
        if self.toWord == relation.to():
            UAS = 1.0
        return ParserEvaluationScore(LAS, UAS, LS, 1)

    def __str__(self) -> str:
        return self.__universalDependencyType.name



from __future__ import annotations
from Corpus.Sentence import Sentence
from DependencyParser.ParserEvaluationScore import ParserEvaluationScore
from DependencyParser.Universal.UniversalDependencyRelation import UniversalDependencyRelation
from DependencyParser.Universal.UniversalDependencyTreeBankFeatures import UniversalDependencyTreeBankFeatures
from DependencyParser.Universal.UniversalDependencyTreeBankWord import UniversalDependencyTreeBankWord
import re


class UniversalDependencyTreeBankSentence(Sentence):

    comments: list

    def __init__(self, sentence: str = None):
        super().__init__()
        self.comments = []
        if sentence is not None:
            lines = sentence.split("\n")
            for line in lines:
                if len(line) == 0:
                    continue
                if line.startswith("#"):
                    self.addComment(line.strip())
                else:
                    items = line.split("\t")
                    if len(items) != 10:
                        print("Line does not contain 10 items ->" + line)
                    else:
                        id = items[0]
                        if re.fullmatch("\\d+", id):
                            surfaceForm = items[1]
                            lemma = items[2]
                            upos = UniversalDependencyRelation.getDependencyPosType(items[3])
                            if upos is None:
                                print("Line does not contain universal pos ->" + line)
                            xpos = items[4]
                            features = UniversalDependencyTreeBankFeatures(items[5])
                            if items[6] != "_":
                                to = int(items[6])
                                dependencyType = items[7].upper()
                                relation = UniversalDependencyRelation(to, dependencyType)
                            else:
                                relation = None
                            deps = items[8]
                            misc = items[9]
                            word = UniversalDependencyTreeBankWord(int(id), surfaceForm, lemma, upos, xpos, features,
                                                               relation, deps, misc)
                            self.addWord(word)

    def addComment(self, comment: str):
        self.comments.append(comment)

    def __str__(self) -> str:
        result = ""
        for comment in self.comments:
            result += comment + "\n"
        for word in self.words:
            result += word.__str__() + "\n"
        return result

    def compareParses(self, sentence: UniversalDependencyTreeBankSentence) -> ParserEvaluationScore:
        score = ParserEvaluationScore()
        for i in range(len(self.words)):
            relation1 = self.words[i].getRelation()
            relation2 = sentence.getWord(i).getRelation()
            if relation1 is not None and relation2 is not None:
                score.add(relation1.compareRelations(relation2))
        return score



from __future__ import annotations

import xml

from Corpus.Corpus import Corpus

from DependencyParser.Turkish.TurkishDependencyTreeBankSentence import TurkishDependencyTreeBankSentence


class TurkishDependencyTreeBankCorpus(Corpus):

    def __init__(self, fileName: str = None):
        """
        Empty constructor for TurkishDependencyTreeBankCorpus. Initializes the sentences and wordList attributes.
        """
        super().__init__()
        if fileName is not None:
            root = xml.etree.ElementTree.parse(fileName).getroot()
            for sentenceNode in root:
                sentence = TurkishDependencyTreeBankSentence(sentenceNode)
                self.sentences.append(sentence)

    def emptyCorpus(self) -> TurkishDependencyTreeBankCorpus:
        """
        Constructor to create an empty copy of this corpus.

        RETURNS
        -------
        TurkishDependencyTreeBankCorpus
            An empty copy of this corpus.
        """
        return TurkishDependencyTreeBankCorpus()



from enum import Enum, auto


class UniversalDependencyPosType(Enum):
    
    ADJ = auto()
    ADV = auto()
    INTJ = auto()
    NOUN = auto()
    PROPN = auto()
    VERB = auto()
    ADP = auto()
    AUX = auto()
    CCONJ = auto()
    DET = auto()
    NUM = auto()
    PART = auto()
    PRON = auto()
    SCONJ = auto()
    PUNCT = auto()
    SYM = auto()
    X = auto()



from xml.etree.ElementTree import Element

from Corpus.Sentence import Sentence

from DependencyParser.Turkish.TurkishDependencyTreeBankWord import TurkishDependencyTreeBankWord


class TurkishDependencyTreeBankSentence(Sentence):

    def __init__(self, sentenceNode: Element):
        """
        Given the parsed xml node which contains information about a sentence, the method constructs a
        TurkishDependencyTreeBankSentence from it.

        PARAMETERS
        ----------
        sentenceNode : Element
            Xml parsed node containing information about a sentence.
        """
        super().__init__()
        for wordNode in sentenceNode:
            word = TurkishDependencyTreeBankWord(wordNode)
            self.words.append(word)

    def maxDependencyLength(self) -> int:
        """
        Calculates the maximum of all word to related word distances, where the distances are calculated in terms of
        index differences.

        RETURNS
        -------
        int
            Maximum of all word to related word distances.
        """
        maxLength = 0
        for i in range(len(self.words)):
            word = self.words[i]
            if word.getRelation() is not None and word.getRelation().to() - i > maxLength:
                maxLength = word.getRelation().to() - i
        return maxLength



from DependencyParser.DependencyRelation import DependencyRelation
from DependencyParser.Turkish.TurkishDependencyType import TurkishDependencyType


class TurkishDependencyRelation(DependencyRelation):

    __toIG: int
    __turkishDependencyType: TurkishDependencyType

    turkishDependencyTypes = ["VOCATIVE", "SUBJECT", "DATIVE.ADJUNCT", "OBJECT", "POSSESSOR",
                              "MODIFIER", "S.MODIFIER", "ABLATIVE.ADJUNCT", "DETERMINER", "SENTENCE",
                              "CLASSIFIER", "LOCATIVE.ADJUNCT", "COORDINATION", "QUESTION.PARTICLE", "INTENSIFIER",
                              "INSTRUMENTAL.ADJUNCT", "RELATIVIZER", "NEGATIVE.PARTICLE", "ETOL", "COLLOCATION",
                              "FOCUS.PARTICLE", "EQU.ADJUNCT", "APPOSITION"]

    turkishDependencyTags = [TurkishDependencyType.VOCATIVE, TurkishDependencyType.SUBJECT,
                             TurkishDependencyType.DATIVE_ADJUNCT, TurkishDependencyType.OBJECT,
                             TurkishDependencyType.POSSESSOR,
                             TurkishDependencyType.MODIFIER, TurkishDependencyType.S_MODIFIER,
                             TurkishDependencyType.ABLATIVE_ADJUNCT, TurkishDependencyType.DETERMINER,
                             TurkishDependencyType.SENTENCE,
                             TurkishDependencyType.CLASSIFIER, TurkishDependencyType.LOCATIVE_ADJUNCT,
                             TurkishDependencyType.COORDINATION, TurkishDependencyType.QUESTION_PARTICLE,
                             TurkishDependencyType.INTENSIFIER,
                             TurkishDependencyType.INSTRUMENTAL_ADJUNCT, TurkishDependencyType.RELATIVIZER,
                             TurkishDependencyType.NEGATIVE_PARTICLE, TurkishDependencyType.ETOL,
                             TurkishDependencyType.COLLOCATION,
                             TurkishDependencyType.FOCUS_PARTICLE, TurkishDependencyType.EQU_ADJUNCT,
                             TurkishDependencyType.APPOSITION]

    @staticmethod
    def getDependencyTag(tag: str) -> TurkishDependencyType:
        """
        The getDependencyTag method takes an dependency tag as string and returns the TurkishDependencyType
        form of it.

        PARAMETERS
        ----------
        tag : str
            Type of the dependency tag in string form

        RETURNS
        -------
        TurkishDependencyType
            Type of the dependency in TurkishDependencyType form
        """
        for j in range(len(TurkishDependencyRelation.turkishDependencyTypes)):
            if tag.upper() == TurkishDependencyRelation.turkishDependencyTypes[j]:
                return TurkishDependencyRelation.turkishDependencyTags[j]
        return None

    def __init__(self, toWord: int, toIG: int, dependencyType: str):
        """
        Another constructor for TurkishDependencyRelation. Gets input toWord, toIG, and dependencyType as arguments and
        calls the super class's constructor and sets the IG and dependency type.

        PARAMETERS
        ----------
        toWord : int
            Index of the word in the sentence that dependency relation is related
        toIG : int
            Index of the inflectional group the dependency relation is related
        dependencyType : str
            Type of the dependency relation in string form
        """
        super().__init__(toWord)
        self.__toIG = toIG
        self.__turkishDependencyType = TurkishDependencyRelation.getDependencyTag(dependencyType)

    def toIG(self) -> int:
        """
        Accessor for the toIG attribute

        RETURNS
        -------
        int
            toIG attribute
        """
        return self.__toIG

    def getTurkishDependencyType(self) -> TurkishDependencyType:
        """
        Accessor for the turkishDependencyType attribute

        RETURNS
        -------
        TurkishDependencyType
            turkishDependencyType attribute
        """
        return self.__turkishDependencyType

    def __str__(self) -> str:
        return self.__turkishDependencyType.name



from enum import Enum, auto


class UniversalDependencyType(Enum):

    ACL = auto()
    ADVCL = auto()
    ADVMOD = auto()
    AMOD = auto()
    APPOS = auto()
    AUX = auto()
    AUXPASS = auto()
    CASE = auto()
    CC = auto()
    CCOMP = auto()
    CLF = auto()
    COMPOUND = auto()
    CONJ = auto()
    COP = auto()
    CSUBJ = auto()
    DEP = auto()
    DET = auto()
    DISCOURSE = auto()
    DISLOCATED = auto()
    EXPL = auto()
    FIXED = auto()
    FLAT = auto()
    FOREIGN = auto()
    GOESWITH = auto()
    IOBJ = auto()
    LIST = auto()
    MARK = auto()
    NMOD = auto()
    NSUBJ = auto()
    NUMMOD = auto()
    OBJ = auto()
    OBL = auto()
    ORPHAN = auto()
    PARATAXIS = auto()
    PUNCT = auto()
    REPARANDUM = auto()
    ROOT = auto()
    VOCATIVE = auto()
    XCOMP = auto()
    ACL_RELCL = auto()
    AUX_PASS = auto()
    CC_PRECONJ = auto()
    COMPOUND_PRT = auto()
    DET_PREDET = auto()
    FLAT_FOREIGN = auto()
    NSUBJ_PASS = auto()
    CSUBJ_PASS = auto()
    NMOD_NPMOD = auto()
    NMOD_POSS = auto()
    NMOD_TMOD = auto()
    ADVMOD_EMPH = auto()
    AUX_Q = auto()
    COMPOUND_LVC = auto()
    COMPOUND_REDUP = auto()
    CSUBJ_COP = auto()
    NMOD_COMP = auto()
    NMOD_PART = auto()
    NSUBJ_COP = auto()
    OBL_AGENT = auto()
    OBL_TMOD = auto()



from setuptools import setup

from pathlib import Path
this_directory = Path(__file__).parent
long_description = (this_directory / "README.md").read_text(encoding="utf-8")

setup(
    name='NlpToolkit-DependencyParser',
    version='1.0.21',
    packages=['DependencyParser', 'DependencyParser.Turkish', 'DependencyParser.Universal', 'DependencyParser.Stanford'],
    url='https://github.com/StarlangSoftware/TurkishDependencyParser-Py',
    license='',
    author='olcaytaner',
    author_email='olcay.yildiz@ozyegin.edu.tr',
    description='Turkish Dependency Parser',
    install_requires=['NlpToolkit-MorphologicalAnalysis'],
    long_description=long_description,
    long_description_content_type='text/markdown'
)



class DependencyRelation:

    toWord: int

    def __init__(self, toWord: int):
        """
        Constructor for a DependencyRelation. Takes toWord as a parameter and sets the corresponding attribute.

        PARAMETERS
        ----------
        toWord : int
            Index of the word in the sentence that dependency relation is related
        """
        self.toWord = toWord

    def to(self) -> int:
        """
        Accessor for toWord attribute

        RETURNS
        -------
        int
            toWord attribute value
        """
        return self.toWord



from enum import Enum, auto


class StanfordDependencyType(Enum):
    ACOMP = auto()
    ADVCL = auto()
    ADVMOD = auto()
    AGENT = auto()
    AMOD = auto()
    APPOS = auto()
    AUX = auto()
    AUXPASS = auto()
    CC = auto()
    CCOMP = auto()
    CONJ = auto()
    COP = auto()
    CSUBJ = auto()
    CSUBJPASS = auto()
    DEP = auto()
    DET = auto()
    DISCOURSE = auto()
    DOBJ = auto()
    EXPL = auto()
    GOESWITH = auto()
    IOBJ = auto()
    MARK = auto()
    MWE = auto()
    NEG = auto()
    NN = auto()
    NPADVMOD = auto()
    NSUBJ = auto()
    NSUBJPASS = auto()
    NUM = auto()
    NUMBER = auto()
    PARATAXIS = auto()
    PCOMP = auto()
    POBJ = auto()
    POSS = auto()
    POSSESSIVE = auto()
    PRECONJ = auto()
    PREDET = auto()
    PREP = auto()
    PREPC = auto()
    PRT = auto()
    PUNCT = auto()
    QUANTMOD = auto()
    RCMOD = auto()
    REF = auto()
    ROOT = auto()
    TMOD = auto()
    VMOD = auto()
    XCOMP = auto()
    XSUBJ = auto()



from enum import Enum, auto


class TurkishDependencyType(Enum):
    VOCATIVE = auto()
    SUBJECT = auto()
    DATIVE_ADJUNCT = auto()
    OBJECT = auto()
    POSSESSOR = auto()
    MODIFIER = auto()
    S_MODIFIER = auto()
    ABLATIVE_ADJUNCT = auto()
    DETERMINER = auto()
    SENTENCE = auto()
    CLASSIFIER = auto()
    LOCATIVE_ADJUNCT = auto()
    COORDINATION = auto()
    QUESTION_PARTICLE = auto()
    INTENSIFIER = auto()
    INSTRUMENTAL_ADJUNCT = auto()
    RELATIVIZER = auto()
    NEGATIVE_PARTICLE = auto()
    ETOL = auto()
    COLLOCATION = auto()
    FOCUS_PARTICLE = auto()
    EQU_ADJUNCT = auto()
    APPOSITION = auto()
